@@ -71,7 +71,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
     ),
   ];
 
-our $VERSION = '1.3.7';
+our $VERSION = '1.3.8';
 
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
@@ -180,7 +180,7 @@ sub cache_key {
 
         # for reasons I don't understand, without walking the reference tree,
         # parts of $default are undef
-        Gscan2pdf::Dialog::Scan::my_dumper( $self->{current_scan_options} );
+        Dumper( $self->{current_scan_options} );
 
         # grep the reload triggers from the current options
         for ( @{ $self->{current_scan_options} } ) {
@@ -261,7 +261,6 @@ sub scan_options {
             my ($options) = @_;
             $pbar->destroy;
             $hboxd->show_all;
-            $logger->info($options);
             if ( $self->get('cache-options') ) {
                 my $cache = $self->get('options-cache');
 
@@ -696,7 +695,7 @@ sub hide_geometry {
 # and walking the options tree, update the widgets
 
 sub set_option {
-    my ( $self, $option, $val, $finished_callback ) = @_;
+    my ( $self, $option, $val ) = @_;
     $option->{val} = $val;
     $self->update_widget( $option->{name}, $val );
 
@@ -841,7 +840,6 @@ sub set_option {
 
         $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
     }
-    if ( defined $finished_callback ) { $finished_callback->() }
     return;
 }
 
@@ -854,7 +852,7 @@ sub patch_cache {
     # for reasons I don't understand, without walking the
     # reference tree, parts of $self->{current_scan_options}
     # are undef
-    Gscan2pdf::Dialog::Scan::my_dumper( $self->{current_scan_options} );
+    Dumper( $self->{current_scan_options} );
     for my $hashref ( @{ $self->{current_scan_options} } ) {
         my ( $key, undef ) = each %{$hashref};
         my $updated_option =
@@ -947,7 +945,7 @@ sub map_geometry_names {
 
         # for reasons I don't understand, without walking the reference tree,
         # parts of $profile are undef
-        Gscan2pdf::Dialog::Scan::my_dumper($profile);
+        Dumper($profile);
         my ( $name, $val ) = each %{ $profile->[$i] };
         given ($name) {
             when (SANE_NAME_SCAN_TL_X) {
@@ -995,7 +993,7 @@ sub map_options {
 
         # for reasons I don't understand, without walking the reference tree,
         # parts of $_ are undef
-        Gscan2pdf::Dialog::Scan::my_dumper($_);
+        Dumper($_);
         my ( $key, $val ) = each %{$_};
         if ( $key ne 'Paper size' ) {
             my $opt = $options->by_name($key);
