@@ -24,7 +24,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
     ),
 ];
 
-our $VERSION = '1.3.7';
+our $VERSION = '1.3.8';
 
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
@@ -460,7 +460,7 @@ sub hide_geometry {
 # and walking the options tree, update the widgets
 
 sub set_option {
-    my ( $self, $option, $val, $finished_callback ) = @_;
+    my ( $self, $option, $val ) = @_;
     if ( not defined $option ) { return }
 
     $self->add_to_current_scan_options( $option, $val );
@@ -504,7 +504,6 @@ sub set_option {
             }
 
             $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
-            if ( defined $finished_callback ) { $finished_callback->() }
         },
         error_callback => sub {
             my ($message) = @_;
@@ -523,7 +522,7 @@ sub map_geometry_names {
 
         # for reasons I don't understand, without walking the reference tree,
         # parts of $profile are undef
-        Gscan2pdf::Dialog::Scan::my_dumper( $profile->[$i] );
+        Dumper( $profile->[$i] );
         my ( $name, $val ) = each %{ $profile->[$i] };
         given ($name) {
             when ('l') {
